# Amethyst - An experimental Minecraft server written in Elixir.
# Copyright (C) 2024 KodiCraft
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

defmodule Amethyst.ConnectionState.Configuration do
  require Amethyst.ConnectionState.Macros
  alias Amethyst.ConnectionState.Macros

  require Logger

  @moduledoc """
  This module contains the packets and logic for the Configuration state.
  """
  Macros.defpacket_clientbound :cookie_request, 0x00, 767, [identifier: :string]
  Macros.defpacket_clientbound :clientbound_plugin_message, 0x01, 767, [channel: :string, data: :raw]
  Macros.defpacket_clientbound :disconnect, 0x02, 767, [reason: :json]
  Macros.defpacket_clientbound :finish_configuration, 0x03, 767, []
  Macros.defpacket_clientbound :clientbound_keep_alive, 0x04, 767, [id: :long]
  Macros.defpacket_clientbound :ping, 0x05, 767, [id: :int]
  Macros.defpacket_clientbound :reset_chat, 0x06, 767, []
  Macros.defpacket_clientbound :registry_data, 0x07, 767, [
    id: :string,
    entries: {:array, [
      id: :string,
      data: {:optional, :nbt}
    ]}
  ]
  Macros.defpacket_clientbound :remove_resource_pack, 0x08, 767, [
    uuid: {:optional, :uuid}
  ]
  Macros.defpacket_clientbound :add_resource_pack, 0x09, 767, [
    uuid: :uuid,
    url: :string,
    hash: :string,
    forced: :bool,
    prompt_message: {:optional, :string}
  ]
  Macros.defpacket_clientbound :store_cookie, 0x0A, 767, [identifier: :string, payload: :byte_array]
  Macros.defpacket_clientbound :transfer, 0x0B, 767, [host: :string, port: :varint]
  Macros.defpacket_clientbound :feature_flags, 0x0C, 767, [flags: {:array, [flag: :string]}]
  Macros.defpacket_clientbound :update_tags, 0x0D, 767, [
    tags: {:array, [
      registry: :string,
      tags: {:array, [
        name: :string,
        entries: {:array, [id: :varint]}
      ]}
    ]}
  ]
  Macros.defpacket_clientbound :clientbound_known_packs, 0x0E, 767, [
    packs: {:array, [
      nameshape: :string,
      id: :string,
      version: :string
    ]}
  ]
  Macros.defpacket_clientbound :custom_report_details, 0x0F, 767, [
    details: {:array, [
      title: :string,
      desctioption: :string
    ]}
  ]
  Macros.defpacket_clientbound :server_links, 0x10, 767, [
    links: {:array, [
      is_builtin: :bool,
      label: :string,
      url: :string
    ]}
  ]

  Macros.defpacket_serverbound :client_information, 0x00, 767, [
    locale: :string,
    view_distance: :byte,
    chat_mode: :varint,
    chat_colors: :bool,
    displayed_skin_parts: :byte,
    main_hand: :varint,
    text_filtering: :bool,
    allow_server_listings: :bool
  ]
  Macros.defpacket_serverbound :cookie_response, 0x01, 767, [
    key: :string,
    payload: {:optional, :byte_array}
  ]
  Macros.defpacket_serverbound :serverbound_plugin_message, 0x02, 767, [channel: :string, data: :raw]
  Macros.defpacket_serverbound :acknowledge_finish_configuration, 0x03, 767, []
  Macros.defpacket_serverbound :serverbound_keep_alive, 0x04, 767, [id: :long]
  Macros.defpacket_serverbound :pong, 0x05, 767, [id: :int]
  Macros.defpacket_serverbound :resource_pack_response, 0x06, 767, [uuid: :uuid, result: :varint]
  Macros.defpacket_serverbound :serverbound_known_packs, 0x07, 767, [
    packs: {:array, [
      namespace: :string,
      id: :string,
      version: :string
    ]}
  ]

  def disconnect(reason) do
    %{packet_type: :disconnect, reason: %{
      "text" => reason
    }}
  end
end
